@@ -53,6 +53,7 @@ SEXP factorR (SEXP n)
   }
   return bigintegerR::create_SEXP(result);
   }    catch(std::invalid_argument & e){
-    error("%s",e.what());
+    Rf_error("%s",e.what());
+    return Rf_mkString(0);
   }
 }
